/*************************************************************************
* fwSDLApp - Application class for SDL
* Author: Thomas Seban
* Date:   17.06.2006
*
* Simple class for a sdl main loop
*************************************************************************/

#ifndef _FWSDLAPP_H_
#define _FWSDLAPP_H_

#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>

using namespace std;

#define MAXFRAMETIME 16

class SDLApp {
  protected:
    const SDL_VideoInfo *mVideoInfo;
    Uint32       mSDLFlags;
    SDL_Surface *mScreen;
    Uint32       mVideoFlags;
    Uint32       mFrames;
    Uint8       *mKeyStates;

    bool         mUseTTFont;
    bool         mDone;
    Uint32       mFPSTicks;
    float        mFPS;
    Uint32       mLastFrameTime;
    Uint32       mMaxFrameTime;

    void gp2xConvertKey(Uint8 *key_table);
    virtual void InitCommandLine() {};

    virtual void InitSDL();
    virtual void InternalInit();
    virtual void InternalShutdown();

    virtual void ClearScreen();
    virtual void FlipScreen();

    virtual void Render();

    virtual void InternalHandleEvent(SDL_Event *event);
    virtual void InternalBeforeDrawScene(SDL_Surface *Screen);
    virtual void InternalDrawScene(SDL_Surface *Screen);
    virtual void InternalAfterDrawScene(SDL_Surface *Screen);
  public:
    SDLApp();
    virtual ~SDLApp();

    virtual void MusicDone() {};
    virtual void LevelDone() {};
    virtual void LevelLoaded() {};

    float getFPS();
    bool KeyPressed(int Key, bool aWaitUntilRelease = false);

    Uint32 RGB(Uint8 r, Uint8 g, Uint8 b);

    SDL_Surface *Screen() { return mScreen; };

    void SetCommandLine(int aArgCount, char* aArgs[]);
    char* GetCommandLineValue(char * aParam);
    bool ExistsParam(char* aParam);

    void Init();
    void Stop();
    void Run();
    Uint32 GetFrameTime() { return mLastFrameTime; };
    SDL_Surface *GetScreen() { return mScreen; };
    void SetScreen(SDL_Surface *aScreen) { mScreen = aScreen; };
};

extern SDLApp* GetApplication();

#endif
