/*************************************************************************
* File:   gp2x.cpp
* Author: Thomas Seban
* Date:   17.06.2006
*
* Accessing gp2x hardware ports
*************************************************************************/

#ifdef GP2X
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <unistd.h>

#include "gp2x.h"

#define SYS_CLK_FREQ 7372800

extern int errno;
volatile unsigned long  *memregs32;
volatile unsigned short *memregs16;
volatile unsigned short *gp2x_memregs = memregs16;

int memfd;

void *trymmap (void *start, size_t length, int prot, int flags, int fd, off_t offset)
{
	char *p;
	int aa;

	fprintf(stdout, "mmap(%X, %X, %X, %X, %X, %X) ... ", (unsigned int)start, length, prot, flags, fd, (unsigned int)offset);
	p = (char *)mmap (start, length, prot, flags, fd, offset);
	if (p == (char *)0xFFFFFFFF)
	{
		aa = errno;
		fprintf(stderr, "failed. errno = %d\n", aa);
	}
	else
	{
		fprintf(stdout, "OK! (%X)\n", (unsigned int)p);
	}

	return p;
}

bool initphys (void)
{
	memfd = open("/dev/mem", O_RDWR);
	if (memfd == -1)
	{
		fprintf (stderr, "Open failed\n");
		return false;
	}

	fprintf(stdout, "/dev/mem opened successfully - fd = %d\n", memfd);

	memregs32 = (unsigned long *)trymmap(0, 0x10000, PROT_READ, MAP_SHARED, memfd, 0xc0000000);
	if (memregs32 == (unsigned long *)0xFFFFFFFF) return 0;

	memregs16 = (unsigned short *)memregs32;

	return true;
}

void closephys (void)
{
	close (memfd);
}

#ifdef DEBUG
unsigned short lastjoy = 0;
unsigned short lastbutton = 0;
unsigned short lastvolume = 0;
#endif

bool gp2x_JoystickPressed(int aButton, bool aSmallRange) {
    unsigned short joy;

    joy = ~memregs16[GPIOMPINLVL >> 1];
#ifdef DEBUG
    if (joy != lastjoy) {
      fprintf(stdout, "joystick: %04X, aButton: %04X\n", joy, aButton);
      lastjoy = joy;
    }
#endif

    if (aButton == gp2x_JoyLeft) {
        if (joy == 0xFE04) return true;
        if (joy == 0xFE06) return true;
        if (joy == 0xFE0C) return true;
        if (!aSmallRange) {
          if (joy == 0xFE02) return true;
          if (joy == 0xFE08) return true;
        }
    }

    if (aButton == gp2x_JoyUp) {
        if (joy == 0xFE01) return true;
        if (joy == 0xFE03) return true;
        if (joy == 0xFE81) return true;
        if (!aSmallRange) {
          if (joy == 0xFE02) return true;
          if (joy == 0xFE80) return true;
        }
    }

    if (aButton == gp2x_JoyRight) {
        if (joy == 0xFE40) return true;
        if (joy == 0xFE60) return true;
        if (joy == 0xFEC0) return true;
        if (!aSmallRange) {
          if (joy == 0xFE20) return true;
          if (joy == 0xFE80) return true;
        }
    }

    if (aButton == gp2x_JoyDown) {
        if (joy == 0xFE10) return true;
        if (joy == 0xFE30) return true;
        if (joy == 0xFE18) return true;
        if (!aSmallRange) {
          if (joy == 0xFE08) return true;
          if (joy == 0xFE20) return true;
        }
    }

    return false;
}

bool gp2x_ButtonPressed(int aButton, bool aWaitUntilRelease) {
    unsigned short button;

    button = ~memregs16[GPIOCPINLVL >> 1];
#ifdef DEBUG
    if (button != lastbutton) {
      fprintf(stdout, "Button: %04X, aButton: %04X\n", button, aButton);
      lastbutton = button;
    }
#endif
    switch (aButton) {
        case gp2x_Start   : return ((button & 0x0100) == 0x0100); break;
        case gp2x_Select  : return ((button & 0x0200) == 0x0200); break;
        case gp2x_LButton : return ((button & 0x0400) == 0x0400); break;
        case gp2x_RButton : return ((button & 0x0800) == 0x0800); break;
        case gp2x_A       : return ((button & 0x1000) == 0x1000); break;
        case gp2x_B       : return ((button & 0x2000) == 0x2000); break;
        case gp2x_X       : return ((button & 0x4000) == 0x4000); break;
        case gp2x_Y       : return ((button & 0x8000) == 0x8000); break;
        default: return 0;
    }

    if (aWaitUntilRelease) {
        while (gp2x_ButtonPressed(aButton)) {
            sleep(1);
        }
    }
}

bool gp2x_VolumePressed(int aButton) {
    unsigned short button;

    button = ~memregs16[GPIODPINLVL >> 1];
#ifdef DEBUG
    if (button != lastvolume) {
      fprintf(stdout, "Volume: %04X, aButton: %04X\n", button, aButton);
      lastvolume = button;
    }
#endif
    switch (aButton) {
        case gp2x_VolMinus: return (button == 0xC040); break;
        case gp2x_VolPlus : return (button == 0xC080); break;
        default: return 0;
    }
}

void SetGP2XClock(unsigned int MHZ)
{
  unsigned int v;
  unsigned int mdiv;
  unsigned int pdiv = 3;
  unsigned int scale = 0;
  MHZ *= 1000000;
  mdiv = (MHZ * pdiv) / SYS_CLK_FREQ;

  mdiv=((mdiv-8)<<8) & 0xff00;
  pdiv=((pdiv-2)<<2) & 0xfc;
  scale&=3;
  v = mdiv | pdiv | scale;

  unsigned int l = memregs32[0x808>>2];// Get interupt flags
  memregs32[0x808>>2] = 0xFF8FFFE7;   //Turn off interrupts
  memregs16[0x910>>1]=v;              //Set frequentie
  while(memregs16[0x0902>>1] & 1);    //Wait for the frequentie to be ajused
  memregs32[0x808>>2] = l;            //Turn on interrupts
}

void mmuunhack(void) {
    system("/sbin/rmmod mmuhack");
}

int mmuhack(void) {
    int mmufd;
    system("/sbin/rmmod mmuhack");
    system("/sbin/insmod mmuhack.o");
    mmufd = open("/dev/mmuhack", 2);
    if(mmufd < 0) return 0;

    close(mmufd);

    atexit(mmuunhack);
    return 1;
}

#else // #ifdef GP2X

unsigned char initphys (void) { return 1; };
void closephys (void) { };
bool gp2x_JoystickPressed(int aButton) { return false; };
bool gp2x_ButtonPressed(int aButton) { return false; };
bool gp2x_VolumePressed(int aButton) { return false; };

int mmuhack(void) { return 1; };
int mmuunhack(void) { return 1; };


#endif // #ifdef GP2X

